from unittest.mock import patch, Mock
import pytest
from mailinglist.hooks import MailinglistDefaultHookset
from django.core.exceptions import ValidationError


class FileNameyThingy:
    def __init__(self, name):
        self.name = name


class TestMailinglistDefaultHookset:
    @patch("mailinglist.hooks.EmailMultiAlternatives")
    def test_send_message(self, p_email_alternatives, message_attachment):
        _message = Mock()
        p_email_alternatives.return_value = _message
        MailinglistDefaultHookset().send_message(
            to="someone@email.com",
            body="good strong body here!",
            subject="[mailinglist] subject or whatever",
            from_email="loving@it.com",
            html_body="good <strong>strong</strong> body here!",
            attachments=[message_attachment],
            headers={"header": "yes"},
        )
        p_email_alternatives.assert_called_once_with(
            to="someone@email.com",
            body="good strong body here!",
            subject="[mailinglist] subject or whatever",
            from_email="loving@it.com",
            headers={"header": "yes"},
        )
        _message.attach_file.assert_called_once_with(message_attachment.file.path)
        _message.attach_alternative.assert_called_once_with(
            "good <strong>strong</strong> body here!", "text/html"
        )
        _message.send.assert_called_once_with()

    @patch("mailinglist.hooks.EmailMultiAlternatives")
    def test_send_message_no_attachments(self, p_email_alternatives):
        _message = Mock()
        p_email_alternatives.return_value = _message
        MailinglistDefaultHookset().send_message(
            to="someone@email.com",
            body="good strong body here!",
            subject="[mailinglist] subject or whatever",
            from_email="loving@it.com",
            html_body="good <strong>strong</strong> body here!",
            headers={"header": "yes"},
        )
        p_email_alternatives.assert_called_once_with(
            to="someone@email.com",
            body="good strong body here!",
            subject="[mailinglist] subject or whatever",
            from_email="loving@it.com",
            headers={"header": "yes"},
        )
        _message.attach_file.assert_not_called()
        _message.attach_alternative.assert_called_once_with(
            "good <strong>strong</strong> body here!", "text/html"
        )
        _message.send.assert_called_once_with()

    @patch("mailinglist.hooks.EmailMultiAlternatives")
    def test_send_message_no_html(self, p_email_alternatives, message_attachment):
        _message = Mock()
        p_email_alternatives.return_value = _message
        MailinglistDefaultHookset().send_message(
            to="someone@email.com",
            body="good strong body here!",
            subject="[mailinglist] subject or whatever",
            from_email="loving@it.com",
            attachments=[message_attachment],
            headers={"header": "yes"},
        )
        p_email_alternatives.assert_called_once_with(
            to="someone@email.com",
            body="good strong body here!",
            subject="[mailinglist] subject or whatever",
            from_email="loving@it.com",
            headers={"header": "yes"},
        )
        _message.attach_file.assert_called_once_with(message_attachment.file.path)
        _message.attach_alternative.assert_not_called()
        _message.send.assert_called_once_with()

    def test_message_attachment_file_validator_bad(self):
        with pytest.raises(ValidationError):
            MailinglistDefaultHookset().message_attachment_file_validator(
                FileNameyThingy(name="file.exe")
            )

    def test_message_attachment_file_validator_jpg(self):
        MailinglistDefaultHookset().message_attachment_file_validator(
            FileNameyThingy("long/path/t/o/file.jpg")
        )

    def test_message_attachment_file_validator_jpeg(self):
        MailinglistDefaultHookset().message_attachment_file_validator(
            FileNameyThingy("long/path/t/o/file.jpeg")
        )

    def test_message_attachment_file_validator_pdf(self):
        MailinglistDefaultHookset().message_attachment_file_validator(
            FileNameyThingy("long/path/t/o/file.pdf")
        )



import pytest
from mailinglist.conf import load_path_attr, MailinglistAppConf
from django.core.exceptions import ImproperlyConfigured


def test_load_path_attr():
    with pytest.raises(ImproperlyConfigured):
        load_path_attr("poopy.butthole")


def test_load_path_attr_other():
    with pytest.raises(ImproperlyConfigured):
        load_path_attr("mailinglist.butthole")


class TestMailinglistAppConf:
    def test_configure_default_sender_email(self):
        assert (
            MailinglistAppConf().configure_default_sender_email("something")
            == "something"
        )

    def test_configure_default_sender_email_fails(self):
        with pytest.raises(ImproperlyConfigured):
            MailinglistAppConf().configure_default_sender_email(None)

    def test_configure_base_url(self):
        assert MailinglistAppConf().configure_base_url("something") == "something"

    def test_configure_base_url_fails(self):
        with pytest.raises(ImproperlyConfigured):
            MailinglistAppConf().configure_base_url(None)
